"""
Summary: This is a test app to demonstrate a bug in Dash where a callback with input dcc.Checklist.value is triggered
by changes to that same checklist's options property.

Expected behaviour (works properly in Dash 2.7.0):
    1. Searching for a city in the search bar should update the checklist's options property, but should NOT trigger
       the callback that updates the increment and last selected city displays (i.e. should not trigger any inputs for
       dcc.Checklist.value)
    2. Selecting a city from the checklist SHOULD trigger the callback that updates the increment and last selected city

Actual behaviour (in Dash 2.8.0 and 2.8.1):
    1. Searching for a city in the search bar does update the checklist's currently displayed options, but also
       increments the counter each time a character is typed (i.e. triggers inputs for dcc.Checklist.value)
"""

import dash
from dash import dcc, html
from dash.dependencies import Input, Output, State
from dash.exceptions import PreventUpdate

app = dash.Dash(__name__, suppress_callback_exceptions=True)
app.title = "Dash Bug Test"

all_city_options = [
    {'label': 'New York City', 'value': 'NYC'},
    {'label': 'Montréal', 'value': 'MTL'},
    {'label': 'San Francisco', 'value': 'SF'},
    {'label': 'London', 'value': 'LON'},
    {'label': 'Berlin', 'value': 'BER'},
    {'label': 'Paris', 'value': 'PAR'},
]

layout = [
    html.Div([
        dcc.Store('callback-count', data=0),
        dcc.Store('all-city-options', data=all_city_options),
        dcc.Store('current-options', data=[]),
        dcc.Input(id='search_input', type='text', placeholder='Search for a city',
                  style={'height': '20px', 'margin-bottom': '10px'}),
        dcc.Checklist(
            id='checklist',
            options=all_city_options
        ),
        html.Div([
            html.Span("This should only increment when selected city changes below:",
                      style={'display': 'inline-block'}),
            html.Div(id='increment-display', style={'display': 'inline-block', 'margin-left': '10px'}),
        ], style={'margin-top': '20px'}),
        html.Div([
            html.Span('Last selected city: ', style={'display': 'inline-block'}),
            html.Div(id='last-selected-city', style={'display': 'inline-block', 'margin-left': '10px'}),
        ]),
    ], style={'padding': '4px 8px'})
]


@app.callback(
    Output('checklist', 'options'),
    Output('current-options', 'data'),
    Input('search_input', 'value'),
    State('checklist', 'options'),
    State('all-city-options', 'data'),
)
def update_checklist_options(search_input, current_options, all_city_options):
    triggered = dash.callback_context.triggered[0]['prop_id']

    if 'search_input' in triggered:
        matches = [option for option in all_city_options if search_input.lower() in option['label'].lower()]
        return matches, current_options

    raise PreventUpdate


@app.callback(
    Output('callback-count', 'data'),
    Output('increment-display', 'children'),
    Output('last-selected-city', 'children'),
    Input('checklist', 'value'),
    State('callback-count', 'data'),
    prevent_initial_call=True
)
def display_last_selected_city(value, increment_value):
    increment_value += 1

    return increment_value, increment_value, value

app.layout = html.Div(layout)

if __name__ == '__main__':
    app.run_server(debug=True)
